/*
	*
	* Copyright 2019 Britanicus <marcusbritanicus@gmail.com>
	*
	* This file is a part of DesQ project ( https://gitlab.com/DesQ/ )
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* (at your option) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#include "PowerManager.hpp"
#include "PowerManagerTray.hpp"

DesQPowerManager::DesQPowerManager() {

    QDBusConnection::sessionBus().connect(
        "org.DesQ.Power",
        "/org/DesQ/Power",
        "org.DesQ.Power",
        "LidOpened",
        this,
        SLOT( handleLidOpened() )
    );

    QDBusConnection::sessionBus().connect(
        "org.DesQ.Power",
        "/org/DesQ/Power",
        "org.DesQ.Power",
        "LidClosed",
        this,
        SLOT( handleLidClosed() )
    );

    QDBusConnection::sessionBus().connect(
        "org.DesQ.Power",
        "/org/DesQ/Power",
        "org.DesQ.Power",
        "SwitchedToBattery",
        this,
        SLOT( handleSwitchedToBattery() )
    );

    QDBusConnection::sessionBus().connect(
        "org.DesQ.Power",
        "/org/DesQ/Power",
        "org.DesQ.Power",
        "SwitchedToACPower",
        this,
        SLOT( handleSwitchedToACPower() )
    );

    QDBusConnection::sessionBus().connect(
        "org.DesQ.Power",
        "/org/DesQ/Power",
        "org.DesQ.Power",
        "BatteryChargeChanged",
        "d",
        this,
        SLOT( handleBatteryChargeChanged( double ) )
    );

    QDBusConnection::sessionBus().connect(
        "org.DesQ.Power",
        "/org/DesQ/Power",
        "org.DesQ.Power",
        "BatteryFullyCharged",
        this,
        SLOT( handleBatteryFullyCharged() )
    );

    QDBusConnection::sessionBus().connect(
        "org.DesQ.Power",
        "/org/DesQ/Power",
        "org.DesQ.Power",
        "BatteryNearlyEmpty",
        this,
        SLOT( handleBatteryNearlyEmpty() )
    );

    QDBusConnection::sessionBus().connect(
        "org.DesQ.Power",
        "/org/DesQ/Power",
        "org.DesQ.Power",
        "BatteryEmpty",
        this,
        SLOT( handleBatteryEmpty() )
    );

    desqPower = new QDBusInterface( "org.DesQ.Power", "/org/DesQ/Power", "org.DesQ.Power", QDBusConnection::sessionBus() );

    QDBusReply<bool> replyB = desqPower->call( "OnBattery" );
    onBattery = replyB.value();
};

/* Start the tray icon */
void DesQPowerManager::startTray() {

    tray = new PowerTrayIcon( this );
    tray->show();

    updateBatteryCharge();
};

void DesQPowerManager::updateBatteryCharge() {

    QDBusReply<double> replyD = desqPower->call( "BatteryCharge" );
    handleBatteryChargeChanged( replyD.value() );
};

/* Lid Open/Close Signals */
void DesQPowerManager::handleLidOpened() {
    // Nothing required as of now: Wayfire handles this with swaylock + swayidle
};

void DesQPowerManager::handleLidClosed() {
    // Nothing required as of now: Wayfire handles this with swaylock + swayidle
};

/* Battery Charge/Discharge Signals */
void DesQPowerManager::handleSwitchedToBattery() {
    // 1. Send a notification using notify-send.
    // 2. Set/Restore display brightness
    // 3. Load/Unload power profiles

    qDebug() << "Switched to Battery";
    onBattery = true;

    QProcess::startDetached(
        "notify-send", {
            "-i", "desq", "-a", "DesQ Power Manager",
            "-t", "2000", "AC Disconnected",
            "The AC Power supply has been disconnected. You're running on battery backup."
        }
    );

    updateBatteryCharge();
};

void DesQPowerManager::handleSwitchedToACPower() {
    // 1. Send a notification using notify-send.
    // 2. Set/Restore display brightness
    // 3. Load/Unload power profiles

    qDebug() << "Switched to AC Power";
    onBattery = false;

    QProcess::startDetached(
        "notify-send", {
            "-i", "desq", "-a", "DesQ Power Manager",
            "-t", "2000", "AC Connected",
            "The AC Power supply has been connected. Your battery is charging."
        }
    );

    updateBatteryCharge();
};

/* Battery charge changed */
void DesQPowerManager::handleBatteryChargeChanged( double value ) {
    // When discharging
    //  1. Set the display brightness based on percentage power left.
    //  2. oad/Unload suitable power modules.
    // When charging, change the battery icon.

    qDebug() << "Battery charge:" << value;
    if ( onBattery ) {
        if ( value < 5 )
            tray->setIcon( QIcon( ":/icons/battery-000.png" ) );

        else if ( value < 10 )
            tray->setIcon( QIcon( ":/icons/battery-005.png" ) );

        else if ( value < 20 )
            tray->setIcon( QIcon( ":/icons/battery-010.png" ) );

        else if ( value < 40 )
            tray->setIcon( QIcon( ":/icons/battery-020.png" ) );

        else if ( value < 60 )
            tray->setIcon( QIcon( ":/icons/battery-040.png" ) );

        else if ( value < 80 )
            tray->setIcon( QIcon( ":/icons/battery-060.png" ) );

        else if ( value < 100 )
            tray->setIcon( QIcon( ":/icons/battery-080.png" ) );

        else
            tray->setIcon( QIcon( ":/icons/battery-100.png" ) );
    }

    else {
        if ( value < 5 )
            tray->setIcon( QIcon( ":/icons/battery-000-charging.png" ) );

        else if ( value < 10 )
            tray->setIcon( QIcon( ":/icons/battery-005-charging.png" ) );

        else if ( value < 20 )
            tray->setIcon( QIcon( ":/icons/battery-010-charging.png" ) );

        else if ( value < 40 )
            tray->setIcon( QIcon( ":/icons/battery-020-charging.png" ) );

        else if ( value < 60 )
            tray->setIcon( QIcon( ":/icons/battery-040-charging.png" ) );

        else if ( value < 80 )
            tray->setIcon( QIcon( ":/icons/battery-060-charging.png" ) );

        else if ( value < 100 )
            tray->setIcon( QIcon( ":/icons/battery-080-charging.png" ) );

        else
            tray->setIcon( QIcon( ":/icons/desq-power-manager.png" ) );
    }
};

/* Battery full signal */
void DesQPowerManager::handleBatteryFullyCharged() {
    // Display a notification for the user

    qDebug() << "Charging complete";
    QProcess::startDetached(
        "notify-send", {
            "-i", "desq", "-a", "DesQ Power Manager",
            "-t", "2000", "Battery Charged",
            "Your battery/UPS is completely charged."
        }
    );
};

/* Battery nearly (5m) and almost (1m) empty */
void DesQPowerManager::handleBatteryNearlyEmpty() {
    // Inform the user that around 5 minutes of battery charge remain.

    tray->setIcon( QIcon( ":/icons/battery-005.png" ) );
    QProcess::startDetached(
        "notify-send", {
            "-i", "desq", "-a", "DesQ Power Manager",
            "-t", "2000", "Battery Nearly Empty",
            "Your battery is nearly empty with approximately 5 minutes of charge is remaining. "
            "Save your work immediately and find an alternate source of power supply."
        }
    );
};

void DesQPowerManager::handleBatteryEmpty() {
    // Inform the user that around 1 minute of battery charge remains.
    // Prepare for suspend/shutdown

    tray->setIcon( QIcon( ":/icons/battery-000.png" ) );
    QProcess::startDetached(
        "notify-send", {
            "-i", "desq", "-a", "DesQ Power Manager",
            "-t", "2000", "Battery Critical",
            "Your battery is has under 1 minute of charge is remaining. "
            "Your system will be suspended immediately unless a power supply is connected."
        }
    );
};
